/* --COPYRIGHT--,EPL
 *Copyright (c) 2008 Texas Instruments and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the Eclipse Public License v1.0
 *  which accompanies this distribution, and is available at
 *  http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Contributors:
 *      Texas Instruments - initial implementation
 *
 * --/COPYRIGHT--*/
package xdc.tools.sg.eclipse.ui.views;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.TypedEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.mozilla.javascript.Callable;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.WrapFactory;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

import xdc.tools.sg.eclipse.Plugin;
import config.XsShell;


public class ScriptView extends ViewPart {
    
    public class ItemAction extends Action {
        Item item = null;
        public ItemAction(Item item) {
            super ();
            this.item = item;
            setText(item.getText());
            Image img = item.getImage();
            if (item instanceof ToolItem) {
                setToolTipText(((ToolItem)item).getToolTipText());
            } else {
                setToolTipText(item.getText());
            }
            if (img != null) {
                ImageDescriptor idesc = ImageDescriptor.createFromImage(img);
                this.setImageDescriptor(idesc);
            }

        }
        
        public void runWithEvent(Event event) {
            if ((item != null) && (item.isListening(event.type))) {
                item.notifyListeners(event.type, event);
            }
        }
    }
    
    public class ScriptEvent extends TypedEvent {

        public int type = -1;
        public static final int CDOC_ADD_PAGE = 32;
        public static final int XDCPATH_CHANGED = 33;
        public ScriptEvent(ScriptView source, int type, Object data, Widget widget) {
            super(source);
            this.type = type;
            this.data = data;
            this.time = (int)System.currentTimeMillis();
            this.widget = widget;
        }

        private static final long serialVersionUID = 1728192386545095894L;
    }
    
    
    public ScriptView() {
        super();
    }

    @Override
    public void setInitializationData(IConfigurationElement cfig, String propertyName, Object data) {
        super.setInitializationData(cfig, propertyName, data);

        /* module name is in the configuration data */
        moduleName = (String) data;
    }

    @Override
    public void createPartControl(Composite parent) {
        this.parent = parent;
        /* create the execution context */
        shell = new XsShell();
        shell.setScriptFilename(moduleName);
        
        /* load the module */
        shell.initXsObjects();
        Main = shell.loadModule(moduleName);

        /* delegate the createPartControl method to the script */
        Object obj = Main.get("createPartControl", Main);
        
        IActionBars aBars = getViewSite().getActionBars();
        IMenuManager tbMMgr = aBars.getMenuManager();
        IToolBarManager tbMgr = aBars.getToolBarManager();
        
        dynamicToolBar = new ToolBar(parent, SWT.HORIZONTAL | SWT.SHADOW_OUT);
        dynamicToolBar.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING |
                GridData.FILL_HORIZONTAL));
        
        dynamicViewMenu = getDynamicViewMenu();
        
        if (obj instanceof Callable) {
            Callable createPartControl = (Callable) obj;
            Context context = Context.getCurrentContext();
            WrapFactory wrapper = context.getWrapFactory();

            Scriptable scriptViewPart = wrapper.wrapAsJavaObject(context, shell, this, ScriptView.class);
            Scriptable scriptParent = wrapper.wrapAsJavaObject(context, shell, parent, Composite.class);
            createPartControl.call(context, shell, Main, new Object[] {scriptParent, scriptViewPart});
        }
        MenuItem [] items = dynamicViewMenu.getItems();
        addMenuItems(tbMMgr, items);

        addToolBar(tbMgr, dynamicToolBar);
        dynamicToolBar.setVisible(false);
        updateActionBars();
        scriptViewListeners.add(this);
    }
    
    protected void addMenuItems(IMenuManager mgr, MenuItem[] items) {
        for (int i = 0; i < items.length; i++) {
            if (items[i].isListening(SWT.Selection)) {
                ItemAction s = new ItemAction(items[i]);
                ActionContributionItem ai = new ActionContributionItem(s);
                mgr.add(ai);   
                
            } else if (items[i].getStyle() == SWT.SEPARATOR) {
                mgr.add(new Separator());
            } else {
                MenuManager smm = new MenuManager(items[i].getText());
                mgr.add(smm);  
                Menu m = items[i].getMenu();
                if (m != null) {
                    MenuItem [] si = m.getItems();
                    if (si.length > 0) {
                        addMenuItems(smm, si);
                    }
                }
            
            }
        }
    }

    protected void addToolBar(IToolBarManager mgr, ToolBar toolbar) {
        
        ToolItem items[] = toolbar.getItems();
        for (int i = 0; i < items.length; i++) {
            if (items[i].getStyle() == SWT.SEPARATOR) {
                mgr.add(new Separator());
            } else {
                ItemAction s = new ItemAction(items[i]);
                ActionContributionItem ai = new ActionContributionItem(s);
                mgr.add(ai);   
            }
        }
        Control[] children = toolbar.getChildren();
        for (int i = 0; i < children.length; i++) {
            if (children[i] instanceof ToolBar) {
                ToolBarManager smm = new ToolBarManager((ToolBar)children[i]);
                mgr.add(new Separator());
                ToolBarContributionItem tbc = new ToolBarContributionItem(smm);
                mgr.add(tbc);  
                addToolBar(mgr, (ToolBar)children[i]);
            }
        }
        mgr.update(true);
    }

    public Boolean hasDynamicViewMenu() {
        return dynamicViewMenu!=null && dynamicViewMenu.getItemCount()>0; 
    }

    public Menu getDynamicViewMenu() {
        if (dynamicViewMenu == null) {
            dynamicViewMenu = new Menu(this.parent.getShell());
        }
        return dynamicViewMenu;
        
    }

    public ToolBar getToolBar() {
        return(dynamicToolBar);
    }
    
    protected void updateActionBars() {
        IActionBars aBars = getViewSite().getActionBars();
        IToolBarManager tbMgr = aBars.getToolBarManager();
        IMenuManager tbMMgr = aBars.getMenuManager();
        tbMgr.update(true);
        tbMMgr.update(true);
        aBars.updateActionBars();
    
    }
    @Override
    public void dispose() {
        scriptViewListeners.remove(this);
        super.dispose();
        shell = null;
        if (dynamicViewMenu != null && !dynamicViewMenu.isDisposed()) {
            dynamicViewMenu.dispose();
        }
        if (dynamicToolBar != null && !dynamicToolBar.isDisposed()) {
            dynamicToolBar.dispose();
        }
    }
    
    
    @Override
    public void setFocus() {
    }

    public Scriptable getCDocMain() {
        Scriptable cm = null;
        BundleContext context = Plugin.getDefault().getBundleContext();
        try {
            Bundle bundles[] = context.getBundles();
            Bundle cdoc = null;
            for (int i = 0; i < bundles.length; i++) {
                Bundle b = bundles[i];
                if (b.getSymbolicName().compareTo(CDOC_PLUGIN_ID) == 0) {
                    cdoc = b;
                    if (cdoc.getState() == Bundle.ACTIVE) {
                        IViewPart cdvp = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(CDOC_VIEW_ID);
                        if (cdvp != null) {
                            cm = ((ScriptView)cdvp).getMain();
                        }
                    }
                    break;
                }
            }
        } catch (PartInitException e) {
            e.printStackTrace();
        }
        
        return (cm);
    }
    
    public Scriptable getMain(){
        return (Main);
    }
    
    public void postScriptEvent(ScriptView view, int type, Object data, Widget widget) {
        ScriptEvent event = new ScriptEvent(view, type, data, widget);
        notifyScriptEventListeners(event);
    }
    
    public static void notifyScriptEventListeners(ScriptEvent event) {
        Object[] listeners = scriptViewListeners.getListeners();
        for (int i = 0; i < listeners.length; ++i) {
            ((ScriptView) listeners[i]).handleScriptEvent(event);
        }
        
    }
    
    public void handleScriptEvent(ScriptEvent event) {
        /* delegate the handleScriptEvent method to the script */
        if (Main.has("handleScriptEvent", Main)) {
            Object obj = Main.get("handleScriptEvent", Main);
        
            if (obj instanceof Callable) {
                Callable handleEvent = (Callable) obj;
                Context context = Context.getCurrentContext();
                WrapFactory wrapper = context.getWrapFactory();

                Scriptable scriptEvent = wrapper.wrapAsJavaObject(context, shell, event, ScriptEvent.class);
                handleEvent.call(context, shell, Main, new Object[] {scriptEvent});
            }
        }
        
    }
    
    public void showView(String viewId) {
        try {
            PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().showView(viewId);
        } catch (PartInitException e) {
        }
    }
    
    Scriptable Main = null;
    private String moduleName = null;
    private Composite parent = null;
    private Menu dynamicViewMenu = null;
    private ToolBar dynamicToolBar = null;
    private XsShell shell = null;
    protected static final String CDOC_PLUGIN_ID = "xdc.tools.cdoc.sg";
    protected static final String CDOC_VIEW_ID = CDOC_PLUGIN_ID + ".View";
    protected static ListenerList scriptViewListeners = new ListenerList();

}
